Module pinned_init::ptr
source · [−]Expand description
Module providing a special pointer trait used to transfer owned data and to allow safer transmuation of data without forgetting to change other pointer types.
Pointer types need to implement the trait in this module, if they want to support this library.
The type system is used to enforce as much as possible, but implementors
still need to pay attention, that their type can implemen OwnedUniquePtr<T>.
Traits
A (smart) unique pointer which owns its data (e.g. alloc::boxed::Box).
This pointer provides access to T via DerefMut.