pub unsafe trait ZeroableOption { }Expand description
Marker trait for types that allow Option<Self> to be set to all zeroes in order to write
None to that location.
Safety
The implementer needs to ensure that unsafe impl Zeroable for Option<Self> {} is sound.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".