Trait pinned_init::SafePinnedInit
source · [−]pub trait SafePinnedInit<T: PinnedInit>: Sealed<T> + Sized {
type Initialized;
fn init(self) -> Self::Initialized;
}Expand description
Sealed trait to facilitate safe initialization of the types supported by this crate.
Use this traits Self::init method to initialize the T contained in self.
This trait is implemented only for Pin<P> where P: OwnedUniquePtr<T> , T: PinnedInit.
Required Associated Types
type Initialized
type Initialized
The type that represents the initialized version of self.
Required Methods
fn init(self) -> Self::Initialized
fn init(self) -> Self::Initialized
Initialize the contents of self.